# SPDX-FileCopyrightText: 2019 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bluefruit_connect.quaternion_packet`
====================================================

Bluefruit Connect App Quaternion data packet.

* Author(s): Dan Halbert for Adafruit Industries

"""

from __future__ import annotations

import struct

from ._xyz_packet import _XYZPacket


class QuaternionPacket(_XYZPacket):
    """Device Motion data to describe device attitude. This data is derived
    from Accelerometer, Gyro, and Magnetometer readings."""

    # Use _XYZPacket to handle x, y, z, and add w.

    _FMT_PARSE: str = "<xxffffx"
    PACKET_LENGTH: int = struct.calcsize(_FMT_PARSE)
    # _FMT_CONSTRUCT doesn't include the trailing checksum byte.
    _FMT_CONSTRUCT: str = "<2sffff"
    _TYPE_HEADER: bytes = b"!Q"

    def __init__(self, x: float, y: float, z: float, w: float) -> None:
        """Construct a QuaternionPacket from the given x, y, z, and w float values."""
        super().__init__(x, y, z)
        self._w = w

    def to_bytes(self) -> bytes:
        """Return the bytes needed to send this packet."""
        partial_packet = struct.pack(
            self._FMT_CONSTRUCT, self._TYPE_HEADER, self._x, self._y, self._z, self._w
        )
        return partial_packet + bytes((self.checksum(partial_packet),))

    @property
    def w(self) -> float:
        """The w value."""
        return self._w


# Register this class with the superclass. This allows the user to import only what is needed.
QuaternionPacket.register_packet_type()



# SPDX-FileCopyrightText: 2019 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bluefruit_connect.gyro_packet`
====================================================

Bluefruit Connect App Gyro data packet.

* Author(s): Dan Halbert for Adafruit Industries

"""

from __future__ import annotations

from ._xyz_packet import _XYZPacket


class GyroPacket(_XYZPacket):
    """A packet of x, y, z float values from a gyroscope."""

    # Everything else is handled by _XYZPacket.
    _TYPE_HEADER: bytes = b"!G"


# Register this class with the superclass. This allows the user to import only what is needed.
GyroPacket.register_packet_type()



# SPDX-FileCopyrightText: 2021 Tony Hansen
#
# SPDX-License-Identifier: MIT

"""
`adafruit_bluefruit_connect.raw_text_packet`
====================================================

Bluefruit Connect App raw text data packet.

Note that the raw text data packet is different from those used by the
Controller module (e.g. Accelerometer, Control Pad, and Color Picker).
Those use the bytes "!x" (where x is specific to the type of packet),
followed by data specific to the packet, followed by a checksum.
The UART text sender instead sends the bytes followed by a newline.
There is no length indicator, no checksum, etc. Of course, that also
precludes the use of an "!" at the beginning of the string.

Consequently, this packet type is MUCH simpler than the other packet types.

* Author(s): Tony Hansen

"""

from __future__ import annotations

from .packet import Packet


class RawTextPacket(Packet):
    """A packet containing a text string."""

    _TYPE_HEADER: bytes = b"RT"

    def __init__(self, text: str) -> None:
        """Construct a RawTextPacket from a binary string."""
        if isinstance(text, bytes):
            self._text: str = text.strip()
        else:
            raise ValueError("Text must be a bytes string")

    @property
    def text(self) -> str:
        """Return the text associated with the object."""
        return self._text


# Register this class with the superclass. This allows the user to import only what is needed.
RawTextPacket.register_packet_type()
